--!parent
--!encoding=utf-8

--****************************************************************************
-- @file LSW CORE LIB: CONSOLE
-- @author Valentin Schmidt
-- @version	0.1
-- @requires lib "detacher"
--****************************************************************************

property $
property ancestor
property _prompt
property _fixEOL

----------------------------------------
-- @constructor
----------------------------------------
on new (me)
	$.loadXtras(me.libdir, ["Console.x32"], [])
	me.ancestor = xtra("Console").new()

	$.import("detacher")

	me._prompt = "> "
	me._fixEOL = $.jsEval("(function(){var re=new RegExp('\r\n|\r','g');return function(s){return s.replace(re,'\n');};})()")

	return me
end

----------------------------------------
--
----------------------------------------
on show (me, deactivateCloseButton)
	_player.debugPlaybackEnabled = 0
	$.include(me.libdir&"put.ls")

	-- attach to existing or create new console
	ok = me.consoleAttach()
	if not ok then
		ok = me.consoleCreate(deactivateCloseButton)
	end if
	if not ok then
		_player.alert("Error: No console available")
		_player.quit()
	end if

	me.consoleSetTitle("Console")

	bgCol = 0 -- 0 = Black
	stdinCol = bgCol*16 + 7	-- 7 = White
	stdoutCol = bgCol*16 + 10 -- 10 = Light Green 6
	stderrCol = bgCol*16 + 4 -- 4 = Red
	me.consoleSetColors(stdinCol, stdoutCol, stderrCol)
end

----------------------------------------
--
----------------------------------------
on repl (me, greeting)
	me.show()
	me.consoleSetInputCallback(#slotConsoleInput, me)
	_player.alertHook = me
	if not voidP(greeting) then me.stdOut(greeting&$.LF)
	me.stdOut(me._prompt)
end

----------------------------------------
--
----------------------------------------
on alertHook (me, aMsg, aText, aType)
	case (aType) of
	#alert: return 0
	otherwise:
		me.stdErr(aMsg&": "&aText&$.LF)
		me.stdOut(me._prompt)
		return 1
	end case
end

----------------------------------------
-- @xtraCallback
----------------------------------------
on slotConsoleInput (me, str)
	$.detacher.detach(#run, me, str)
end

----------------------------------------
--
----------------------------------------
on run (me, cmd)
	do(cmd)
	me.stdOut(me._prompt)
end

----------------------------------------
-- console.log(obj1 [, obj2, ..., objN)
-- console.log(msg [, subst1, ..., substN)
----------------------------------------
on log (me)
	cnt = paramCount()
	if cnt<2 then return $.console.stdOut($.LF)
	str = string(param(2))
	repeat with i = 3 to cnt
		put "  "&string(param(i)) after str
	end repeat
	$.console.stdOut(str&$.LF)
end

----------------------------------------
-- console.clear()
----------------------------------------
on clear (me)
	me.consoleClear()
end

----------------------------------------
--
----------------------------------------
on _enc (me, arg)
	case ilk(arg) of
		#symbol: return "#"&arg
		#string:
			arg = me._fixEOL.call(VOID, arg)
			return QUOTE&arg&QUOTE
		otherwise:
			return string(arg)
	end case
end
